@@ -1,7 +1,7 @@
 /*
  * AMBA Plug'n Play vendor and device ID definitions.
  *
- * COPYRIGHT (c) 2023.
+ * COPYRIGHT (c) 2024.
  * Frontgrade Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
  * Any changes/bug-fixes should be made in the script, then rerun the script
  * to create a new file.
  *
- * Created from GRLIB revision 4282
+ * Created from GRLIB revision 4293
  */
 
 #ifndef _ambapp_ids_h_
@@ -62,6 +62,8 @@
 #define VENDOR_JHUAPL_SES    0x27
 #define VENDOR_SEMIBLOCKS    0x28
 #define VENDOR_NEC           0x29
+#define VENDOR_HONEYWELL     0x2a
+#define VENDOR_AITECH        0x30
 #define VENDOR_S3            0x31
 #define VENDOR_TAS           0xa5
 #define VENDOR_RECORE        0xa6
@@ -290,6 +292,9 @@
 #define GAISLER_L2CACHE_IO   0x0d1
 #define GAISLER_AHB2AHB_STR  0x0d2
 #define GAISLER_GRIOMMURV    0x0d3
+#define GAISLER_SGMII_VCU118 0x0d4
+#define GAISLER_APBUART_16550 0x0d5
+#define GAISLER_IOPMP        0x0d6
 
 
 /* ESA
@@ -358,6 +363,7 @@
 #define BSC_CORE1            0x001
 #define BSC_CORE2            0x002
 #define BSC_PMU              0x003
+#define BSC_SAFESU           0x004
 
 
 /* GLEICHMANN
@@ -550,6 +556,8 @@
 #define MICROSEMI_M2GL_EVAL  0x0560
 #define XILINX_SP601         0x0601
 #define XILINX_ML605         0x0605
+#define LEON_LATTICE_NEXUS   0x0800
+#define GAISLER_GR740MINI    0x0801
 #define XILINX_AC701         0xa701
 #define XILINX_KC705         0xa705
 #define XILINX_VC707         0xa707
