@@ -44,6 +44,18 @@ char *prop_str_timeout_sec = "timeout-sec";
 char *prop_str_hw_timeout_ms = "gaisler,max-hw-timeout-ms";
 #endif
 
+/* Linux common clock framework properties */
+
+/* Clock producer properties */
+char *prop_str_clock_cells = "#clock-cells";
+char *prop_str_clock_output_names = "clock-output-names";
+char *prop_str_protected_clocks = "protected-clocks";
+char *prop_str_clock_indices = "clock-indices";
+
+/* Clock consumer properties: */
+char *prop_str_clocks = "clocks";
+char *prop_str_clock_names = "clock-names";
+
 #ifdef CONFIG_CUSTOM_NODES
 /* Example for how to create a custom AMBA node tree. This was tested on a
  * GR712 Evaluation Board at 80MHz.
