@@ -12,6 +12,11 @@
 #include <iommu.h>
 #include <grcg.h>
 #include <gpreg.h>
+#include <grwatchdog.h>
+
+#ifdef CONFIG_WATCHDOG
+#include <prom_watchdog.h>
+#endif
 
 #define REMOVE_NODE 1
 
@@ -19,7 +24,7 @@
 #define GR740_REV0_SYSTEMID 0x07401038
 #define GR740_REV1_SYSTEMID 0x07401039
 
-static inline int amba_systemid_is_gr740(void)
+int amba_systemid_is_gr740(void)
 {
 	return (amba_systemid == GR740_REV0_SYSTEMID) ||
 	       (amba_systemid == GR740_REV1_SYSTEMID);
@@ -125,7 +130,7 @@ int ambapp_prop_str_array(struct prop_std *buf, const char *prop_str,
 			  const char *val_array[]);
 int ambapp_prop_empty(struct prop_std *buf, const char *prop_str);
 
-static int gptimer_inited = 0;
+int gptimer_inited = 0;
 
 /* Fixup IRQ of GPTIMER APBSLV interface */
 int gptimer_apb_fixup(ambapp_dev *dev)
@@ -159,52 +164,34 @@ void gpreg_init(ambapp_core *core, int level)
 		gpreg = (struct gpreg_regs *)core->apbslv->address;
 }
 
-#ifdef CONFIG_WATCHDOG
+#ifdef CONFIG_GPTIMER_WDT
 
-#define GPTIMER_WATCHDOG_GR740_HW_TIMEOUT 2000
-
-void gptimer_add_watchdog_node(struct node *node)
+void node_add_watchdog(struct node *parent, unsigned int ambapp_device_id,
+		       unsigned int corenum)
 {
 	struct node *n;
 	int size;
-	unsigned hw_timeout_ms = 0;
 	unsigned int prop_buf[64];
 	struct prop_std *buf = (struct prop_std *)prop_buf;
+	struct watchdog_prop_conf *prop_conf;
 
 #ifdef HASXML
 	/* Check if there already exist a watchdog node. In that case skip
 	 * adding the node as it most likely has been added from xml
 	 */
-	if (node_has_child(node, prop_str_watchdog, 16, NULL))
+	if (node_has_child(parent, prop_str_watchdog, 16, NULL))
 		return;
 #endif
 
-	/* Set the hw timeout. */
-	if (amba_systemid_is_gr740()) {
-		/* GR740 has a PLL watchdog with a timeout of 2 seconds */
-		hw_timeout_ms = GPTIMER_WATCHDOG_GR740_HW_TIMEOUT;
-	} else {
-		uint32_t config;
-		int ntimers;
-		/* Get the pre-configured reload value
-		 * from the watchdog timer which is assumed to be
-		 * configured on boot.
-		 */
-		config = BYPASS_LOAD_PA(&gptimer->config);
-		ntimers = config & 0x7;
-		hw_timeout_ms =
-			BYPASS_LOAD_PA(&gptimer->timer[ntimers - 1].rld);
-		hw_timeout_ms /= 1000;
-	}
-	/* The hw_timeout_ms needs to be set, otherwise skip. */
-	if (!hw_timeout_ms)
+	prop_conf = watchdog_get_prop_conf(ambapp_device_id, corenum);
+	if (!prop_conf)
 		return;
 
 	/* Create a watchdog node and add it:
 	 * watchdog {
-	 *    compatible = "gaisler,gptimer-wdt";
-	 *    gaisler,max-hw-timeout-ms = 2000;
-	 *    timeout-sec = 10;
+	 *    compatible = "<compatible-string>";
+	 *    gaisler,max-hw-timeout-ms = <milliseconds>;
+	 *    timeout-sec = <seconds>;
 	 *    name = "watchdog";
 	 * };
 	 */
@@ -219,32 +206,35 @@ void gptimer_add_watchdog_node(struct node *node)
 	prop_add_dup(n, buf, size, NULL);
 
 	/* Set the compatible string so the driver can match */
-	size = ambapp_prop_str(buf, prop_str_compatible, "gaisler,gptimer-wdt");
+	size = ambapp_prop_str(buf, prop_str_compatible, prop_conf->compatible);
 	prop_add_dup(n, buf, size, NULL);
 
-#ifdef CONFIG_WATCHDOG_USER_TIMEOUT
-	/* Override the user timeout if it is specified */
-	size = ambapp_prop_int(buf, prop_str_timeout_sec,
-			CONFIG_WATCHDOG_USER_TIMEOUT);
-	prop_add_dup(n, buf, size, NULL);
-#endif
+	/* Write the user-timeout (optional) */
+	if (prop_conf->user_timeout_sec != WATCHDOG_TIMEOUT_NOT_SET) {
+		size = ambapp_prop_int(buf, prop_str_timeout_sec,
+				prop_conf->user_timeout_sec);
+		prop_add_dup(n, buf, size, NULL);
+	}
 
 	/* Set the hw timeout of the watchdog */
-	size = ambapp_prop_int(buf, prop_str_hw_timeout_ms, hw_timeout_ms);
+	size = ambapp_prop_int(buf, prop_str_hw_timeout_ms,
+			prop_conf->hw_timeout_ms);
 	buf->options |= PO_END;
 	prop_add_dup(n, buf, size, NULL);
 
 	/* Finally, add the watchdog node to the parent node */
-	node_add_child(node, n);
+	node_add_child(parent, n);
 }
 
 int gptimer_fixup_node(ambapp_core *core, struct node *node)
 {
 	/* Use static as core subindex is not generally available. */
 	static int numnode = 0;
+	ambapp_dev *dev;
 
+	dev = ambapp_get_first_dev(core);
 	if (numnode == 0)
-		gptimer_add_watchdog_node(node);
+		node_add_watchdog(node, dev->device, numnode);
 
 	numnode++;
 
@@ -723,19 +713,45 @@ int i2c_fixup_prop(ambapp_core *core, struct prop_std *buf, int index)
 {
 	int val;
 	static char *prop_str_regshift = NULL;
+	static char *prop_str_ocores_clock_frequency = NULL;
 
 	switch (index) {
 	case 0:
 		return ambapp_prop_str_array(buf, prop_str_compatible,
 					     i2c_compatibles);
 	case 1:
-		/* Set "clock-frequency" */
+ /* Since Linux 4.0 (3a33a85401ecdb0e2c01ea86d9e36a5711ce01d4) the meaning of
+  * "clock-frequency" has changed.
+  *
+  * For kernel versions >= 4.0, Mklinuximg will generate:
+  * "clock-frequency" = I2C bus frequency, default 100 KHz.
+  * "opencores,ip-clock-frequency" = Input clock frequency of the I2C controller
+  *
+  * For kernel versions < 4.0 Mklinuximg will generate:
+  * "clock-frequency" = Input clock frequency of the I2C controller
+  * Note: The Linux driver (< 4.0) use a hardcoded I2C bus frequency = 100 KHz.
+  */
+#if CONFIG_LINUX_VERSION_CODE >= KERNEL_VER(4, 0, 0)
+		/* I2C bus frequency, default 100 KHz. */
+		val = 100000;
+#else
+		/* Input clock frequency of the I2C controller */
 		val = ambapp_dev_freq((ambapp_dev *)core->apbslv);
+#endif
 		return ambapp_prop_int(buf, prop_str_clockfrequency, val);
 	case 2:
 		/* Set "reg-shift" */
 		PROM_STRDUP_ONCE(prop_str_regshift, "reg-shift");
 		return ambapp_prop_int(buf, prop_str_regshift, 2);
+#if CONFIG_LINUX_VERSION_CODE >= KERNEL_VER(4, 0, 0)
+	case 3:
+		/* Input clock frequency of the I2C controller */
+		PROM_STRDUP_ONCE(prop_str_ocores_clock_frequency,
+				 "opencores,ip-clock-frequency");
+		val = ambapp_dev_freq((ambapp_dev *)core->apbslv);
+		return ambapp_prop_int(buf, prop_str_ocores_clock_frequency,
+				       val);
+#endif
 	default:
 		return 0;
 	}
@@ -1484,6 +1500,18 @@ int grpci2_dma_fixup_node(ambapp_core *core, struct node *node)
 	return ret;
 }
 
+#ifdef CONFIG_GRWATCHDOG_WDT
+
+void grwatchdog_init(ambapp_core *core, int level)
+{
+	if (!grwatchdog_inited && core->apbslv) {
+		grwatchdog = (struct grwatchdog_regs *)core->apbslv->address;
+		grwatchdog_inited = 1;
+	}
+}
+
+#endif
+
 struct amba_driver {
 	unsigned short vendor;
 	unsigned short device;
@@ -1557,6 +1585,10 @@ struct amba_driver amba_drivers[] = {
 			NULL, gpreg_init, NULL),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_GRPCI2_DMA, NULL, NULL,     \
 			NULL, NULL, grpci2_dma_fixup_node),
+#ifdef CONFIG_GRWATCHDOG_WDT
+ AMBAPP_DRV(VENDOR_GAISLER, GAISLER_GRWATCHDOG, NULL, NULL,     \
+			NULL, grwatchdog_init, NULL),
+#endif
 };
 #define AMBAPP_DRV_NUM (sizeof(amba_drivers)/sizeof(struct amba_driver))
 
