@@ -1,7 +1,7 @@
 /*
  * AMBA Plug'n Play vendor and device name definitions.
  *
- * COPYRIGHT (c) 2023.
+ * COPYRIGHT (c) 2024.
  * Frontgrade Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
  * Any changes/bug-fixes should be made in the script, then rerun the script
  * to create a new file.
  *
- * Created from GRLIB revision 4282
+ * Created from GRLIB revision 4293
  */
 
 #include <ambapp_names.h>
@@ -250,7 +250,10 @@ static struct ambapp_device_name GAISLER_devices[] =
   {GAISLER_L2CL, "L2CL", "L2-Cache Controller - Lite"},
   {GAISLER_L2CACHE_IO, "L2CACHE_IO", "L2-Cache Controller I/O Port"},
   {GAISLER_AHB2AHB_STR, "AHB2AHB_STR", "AHB/AHB Stripe Bridge"},
-  {GAISLER_GRIOMMURV, "GRIOMMURV", "RISCV IO Memory Management Unit"},
+  {GAISLER_GRIOMMURV, "GRIOMMURV", "RISC-V IO Memory Management Unit"},
+  {GAISLER_SGMII_VCU118, "SGMII_VCU118", "VCU118 SGMII Interface"},
+  {GAISLER_APBUART_16550, "APBUART_16550", "Generic UART 16550"},
+  {GAISLER_IOPMP, "IOPMP", "RISC-V IOPMP"},
   {0, NULL, NULL}
 };
 
@@ -371,6 +374,7 @@ static struct ambapp_device_name BSC_devices[] =
   {BSC_CORE1, "CORE1", "Core 1"},
   {BSC_CORE2, "CORE2", "Core 2"},
   {BSC_PMU, "PMU", "AHB Performance Monitoring Unit"},
+  {BSC_SAFESU, "SAFESU", "SafeSU - AHB Statistics Unit"},
   {0, NULL, NULL}
 };
 
@@ -516,6 +520,20 @@ static struct ambapp_device_name NEC_devices[] =
 };
 
 
+/* Vendor HONEYWELL devices */
+static struct ambapp_device_name HONEYWELL_devices[] =
+{
+  {0, NULL, NULL}
+};
+
+
+/* Vendor AITECH devices */
+static struct ambapp_device_name AITECH_devices[] =
+{
+  {0, NULL, NULL}
+};
+
+
 /* Vendor S3 devices */
 static struct ambapp_device_name S3_devices[] =
 {
@@ -706,6 +724,8 @@ static struct ambapp_ids vendors[] =
   {VENDOR_JHUAPL_SES, "JHUAPL_SES", "JHUAPL Space Exploration", JHUAPL_SES_devices},
   {VENDOR_SEMIBLOCKS, "SEMIBLOCKS", "SemiBlocks B.V.", SEMIBLOCKS_devices},
   {VENDOR_NEC, "NEC", "NEC Corporation", NEC_devices},
+  {VENDOR_HONEYWELL, "HONEYWELL", "Honeywell International", HONEYWELL_devices},
+  {VENDOR_AITECH, "AITECH", "Aitech Systems", AITECH_devices},
   {VENDOR_S3, "S3", "S3 Group", S3_devices},
   {VENDOR_TAS, "TAS", "Thales Alenia Space", TAS_devices},
   {VENDOR_RECORE, "RECORE", "Recore Systems", RECORE_devices},
